﻿/*
 * Photostand. Forms1.cs
 * Copyright(c) 2008, Shigeki Kaneko. All right reserved.
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.IO;
using System.Runtime.InteropServices;
using System.Collections.ObjectModel;



namespace photostand
{
    public partial class Photostand : Form
    {
        private Wallpaper wallpaper;
        private KeywordSelect keywordselect;
        private OptionSetting optionsetting;
        private bool isActive;

        public Photostand()
        {
            InitializeComponent();

            // My initialize code for the application
            this.wallpaper = new Wallpaper();
            // this.listBox1.Items.Add(@"C:\Users\Public\Pictures\Sample Pictures");
            // this.listBox1.Items.Add(@"C:\tmp");
            // this.folderBrowserDialog1.RootFolder = Environment.SpecialFolder.Personal;

            // setting the wallpaper change is active.
            this.isActive = true;
            this.notifyIcon1.Icon = photostand.Properties.Resources.photostand_active;
            this.button7.Text = Properties.Resources.Suspend;

            // setup the listBox
            foreach (DirectoryInfo di in this.wallpaper.dirList) {
                this.listBox1.Items.Add(di.ToString());
            }
            foreach (string s in this.wallpaper.keywordList) {
                this.listBox1.Items.Add("[KW] " + s);
            }

            // Initialize the comboBox of changing period.
            string[] cb = { Properties.Resources.timePeriod_0, Properties.Resources.timePeriod_1, Properties.Resources.timePeriod_2, Properties.Resources.timePeriod_3, Properties.Resources.timePeriod_4, Properties.Resources.timePeriod_5, Properties.Resources.timePeriod_6, Properties.Resources.timePeriod_7, Properties.Resources.timePeriod_8, Properties.Resources.timePeriod_9, Properties.Resources.timePeriod_10, Properties.Resources.timePeriod_11 };
            this.comboBox1.Items.AddRange(cb);
            this.comboBox1.SelectedIndex = wallpaper.timePeriod;

            // Initialize the checkListBox of day of week.
            string[] cl = { Properties.Resources.d_Sunday, Properties.Resources.d_Monday, Properties.Resources.d_Tuesday, Properties.Resources.d_Wednesday, Properties.Resources.d_Thursday, Properties.Resources.d_Friday, Properties.Resources.d_Saturday };
            this.checkedListBox1.Items.AddRange(cl);

            // Initialize the comboBox of day of week.
            switch (wallpaper.dispositionStyle) {
                case 0:
                    this.radioButton1.Checked = true;
                    break;
                case 1:
                    this.radioButton2.Checked = true;
                    break;
                case 2:
                    this.radioButton3.Checked = true;
                    break;
                case 3:
                    this.radioButton4.Checked = true;
                    break;
                default:
                    this.radioButton5.Checked = true;
                    break;
            }

            // Initialize the comboBox to set ratio of web pictures.
            // this.comboBox3.SelectedIndex = wallpaper.webLocalRatio;
            this.trackBar1.Value = this.wallpaper.webLocalRatio;
            this.label8.Text = String.Format("{0,3:G} %", this.trackBar1.Value * 25);

            // Setting the time to the next chaging time.
            DateTime dt = this.wallpaper.dateLastChanged;
            int min = Photostand.Index2Minutes(this.comboBox1.SelectedIndex);
            if (min > 0) {
                dt = dt.AddMinutes(min);
                if (dt.CompareTo(DateTime.Now) <= 0) {
                    TimerExpired();
                    this.timer1.Interval = min * 60 * 1000;
                } else {
                    this.timer1.Interval = (int)(dt.Subtract(DateTime.Now).Ticks / 10000);
                }
                this.timer1.Start();
            }

            // The application is showed only on the iconbox, when starting.
            // this.Hide();
            this.ShowInTaskbar = false;
            this.WindowState = FormWindowState.Minimized;

            // Put debug message out to event log
            // System.Diagnostics.EventLogTraceListener eventlog = new EventLogTraceListener("Photostand");
            // Debug.Listeners.Add(eventlog);

            // configIsChanged is the bool flag to show the configuration is changed and it should be saved to the file.
            this.wallpaper.configIsChanged = false;

            // to bug fix to show Windows wallpaper after booting.
            if (this.wallpaper.pictureFile.Equals(String.Empty)) {
                this.wallpaper.RandomSetWallpaper();
            } else {
                this.wallpaper.ChangeDirectWallpaper(this.wallpaper.pictureFile);
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            // random selection from the list of all pictures.
            int rt = this.wallpaper.RandomSetWallpaper();
            if (rt == 0) {
                display_PictureChanged(this.wallpaper.displayNameOfPictureFile);
            } else {
                display_PictureChanged(null);
            }
        }

        private void FileQuitXToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DialogResult dr = MessageBox.Show(Properties.Resources.CloseCaution, Properties.Resources.Exit, MessageBoxButtons.YesNo, MessageBoxIcon.Question);
            if (dr == DialogResult.Yes) {
                this.Close();
            }
        }

        private void HelpAboutVToolStripMenuItem_Click(object sender, EventArgs e)
        {
            const string version = "2.4.0";
            MessageBox.Show("PhotoStand wallpaper changer\nVersion Number is " + version + ".\n\nCopyright 2008-2012 Shigeki Kaneko. All right reserved.");
        }

        private void HelpIndexCToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Process.Start(Properties.Resources.HelpURL);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            this.wallpaper.SaveConfig();
            this.toolStripStatusLabel1.Text = Properties.Resources.SettingsSaved;
        }

        private void button3_Click(object sender, EventArgs e)
        {
            string fld;

            // This setting limits the folder to access from the dialog. If you want to set any folder, uncomment the next line.
            this.folderBrowserDialog1.RootFolder = this.wallpaper.folderRoot == 0 ? Environment.SpecialFolder.MyPictures : Environment.SpecialFolder.MyComputer;
            // this.folderBrowserDialog1.RootFolder = Environment.SpecialFolder.MyComputer;
            // this.folderBrowserDialog1.SelectedPath = Environment.SpecialFolder.MyPictures.ToString();
            DialogResult rt = this.folderBrowserDialog1.ShowDialog();
            if (rt == DialogResult.OK) {
                fld = this.folderBrowserDialog1.SelectedPath;
                bool isRegistered = false;
                foreach (string s in this.listBox1.Items) {
                    if (fld == s) {
                        this.toolStripStatusLabel1.Text = fld + Properties.Resources.IsRegistered;
                        isRegistered = true;
                        break;
                    }
                }
                if (!isRegistered) {
                    DirectoryInfo di = new DirectoryInfo(fld);
                    this.listBox1.Items.Add(fld);
                    this.wallpaper.dirList.Add(di);
                    this.wallpaper.configIsChanged = true;
                    this.toolStripStatusLabel1.Text = fld + Properties.Resources.DoRegistered;
                }
            }
        }

        private void button4_Click(object sender, EventArgs e)
        {
            string s;
            for (int i = 0; i < this.listBox1.SelectedIndices.Count; i++) {
                string sl = this.listBox1.Items[this.listBox1.SelectedIndices[i]].ToString();
                if (!sl.StartsWith("[KW] ")) {
                    // this is a directory
                    s = sl.Replace("\\\\", "\\");

                    for (int j = 0; j < this.wallpaper.dirList.Count; j++) {
                        if (this.wallpaper.dirList[j].ToString() == s) {
                            this.wallpaper.dirList.Remove(this.wallpaper.dirList[j]);
                            this.wallpaper.configIsChanged = true;
                        }
                    }
                } else {
                    // this is a keyword
                    s = sl.Remove(0, 5);
                    for (int j = 0; j < this.wallpaper.keywordList.Count; j++) {
                        if (this.wallpaper.keywordList[j].ToString() == s) {
                            this.wallpaper.keywordList.Remove(this.wallpaper.keywordList[j]);
                            this.wallpaper.configIsChanged = true;
                        }
                    }
                }
                this.toolStripStatusLabel1.Text = "\"" + s + "\" " + Properties.Resources.IsRemoved;
                this.listBox1.Items.RemoveAt(this.listBox1.SelectedIndices[i]);
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            this.timer1.Stop();
            TimerExpired();
            int min = Photostand.Index2Minutes(this.comboBox1.SelectedIndex);
            if (min > 0) {
                this.timer1.Interval = min * 60 * 1000;
                this.timer1.Start();
            }
        }

        /// <summary>
        /// Timer call back: Change the wallpaper if the timer is expired.
        /// </summary>
        private void TimerExpired()
        {
            // this.RandomSetWallpaper();
            int rt = this.wallpaper.RandomSetWallpaper();
            if (rt == 0) {
                this.display_PictureChanged(this.wallpaper.displayNameOfPictureFile);
            }
        }

        /// <summary>
        /// "1分おき", "2分おき", "5分おき", "10分おき", "30分おき", "1時間おき", "2時間おき", "6時間おき", "12時間おき", "1日おき", "1週間おき", "変更しない"
        /// </summary>
        /// <param name="index"></param>
        /// <returns>number of minutes corresponding the index. -1 means forever.</returns>
        private static int Index2Minutes(int index)
        {
            int min;
            switch (index) {
                case 0:
                    min = 1; break;
                case 1:
                    min = 2; break;
                case 2:
                    min = 5; break;
                case 3:
                    min = 10; break;
                case 4:
                    min = 30; break;
                case 5:
                    min = 1 * 60; break;
                case 6:
                    min = 2 * 60; break;
                case 7:
                    min = 6 * 60; break;
                case 8:
                    min = 12 * 60; break;
                case 9:
                    min = 1 * 24 * 60; break;
                case 10:
                    min = 7 * 24 * 60; break;
                case 11:
                    min = -1; break;
                default:
                    min = 10; break;
            }
            return min;
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.wallpaper.timePeriod = this.comboBox1.SelectedIndex;
            this.wallpaper.configIsChanged = true;
            int min = Photostand.Index2Minutes(this.comboBox1.SelectedIndex);
            if (min > 0) {
                this.timer1.Interval = min * 60 * 1000;
                this.timer1.Start();
            } else {
                this.timer1.Stop();
            }

            // display status information on the footer.
            this.toolStripStatusLabel1.Text = Properties.Resources.ChangeTimePeriod_0 + this.comboBox1.Items[this.comboBox1.SelectedIndex].ToString() + Properties.Resources.ChangeTimePeriod_1;
        }

        private void notifyIcon1_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            this.ShowInTaskbar = true;
            if (this.WindowState == FormWindowState.Minimized) {
                this.WindowState = FormWindowState.Normal;
            }
            this.Show();
        }

        private void toolStripMenuItem2_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void toolStripMenuItem1_Click(object sender, EventArgs e)
        {
            this.ShowInTaskbar = true;
            if (this.WindowState == FormWindowState.Minimized) {
                this.WindowState = FormWindowState.Normal;
            }
            this.Show();
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.notifyIcon1.Visible = false;
        }

        private void FileCloseToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (this.wallpaper.configIsChanged == true) {
                this.wallpaper.SaveConfig();
                this.toolStripStatusLabel1.Text = Properties.Resources.SettingsSaved;
            }
            this.Hide();
            this.ShowInTaskbar = false;
        }

        private void changeDisplayStyle(int idx)
        {
            if (wallpaper.dispositionStyle == idx) {
                Debug.WriteLine("changeDisplayStyle: value is not changed\n");
                return;
            }
            this.wallpaper.dispositionStyle = idx;
            this.wallpaper.configIsChanged = true;

            // Repaint the picture wirh the new display style.
            int rt = this.wallpaper.ChangeWallpaper();
            if (rt == 0) {
                RadioButton rb;
                switch (idx) {
                    case 0:
                        rb = this.radioButton1;
                        break;
                    case 1:
                        rb = this.radioButton2;
                        break;
                    case 2:
                        rb = this.radioButton3;
                        break;
                    case 3:
                        rb = this.radioButton4;
                        break;
                    default:
                        rb = this.radioButton5;
                        break;
                }
                this.toolStripStatusLabel1.Text = Properties.Resources.SetPosition_0 + rb.Text + Properties.Resources.SetPosition_1;
            } else {
                this.toolStripStatusLabel1.Text = Properties.Resources.SetPosition_NG;
            }
        }

        private void button6_Click(object sender, EventArgs e)
        {
            keywordselect = new KeywordSelect();
            keywordselect.Show();
            keywordselect.FormClosing += new FormClosingEventHandler(this.keywordselect_FormClosing);
        }

        private void keywordselect_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (this.keywordselect.checkedKeywords == null) {
                Debug.WriteLine("keywordselect_FormClosing: this.keywordselect.checkedKeywords is null");
                return;
            }
            Debug.WriteLine("keywordselect_FormClosing:" + this.keywordselect.checkedKeywords.Count);
            if (this.keywordselect.checkedKeywords.Count <= 0) {
                Debug.WriteLine("keywordselect_FormClosing: keyword is not set.\n");
                return;
            }

            foreach (string kwd in this.keywordselect.checkedKeywords) {
                string ck = "[KW] " + kwd;
                bool isRegistered = false;
                foreach (string s in this.listBox1.Items) {
                    if (ck.Equals(s)) {
                        this.toolStripStatusLabel1.Text = "\"" + ck + Properties.Resources.IsRegistered;
                        isRegistered = true;
                        break;
                    }
                }
                if (!isRegistered) {
                    this.listBox1.Items.Add(ck);
                    this.wallpaper.keywordList.Add(kwd);
                    this.wallpaper.configIsChanged = true;
                    this.toolStripStatusLabel1.Text = "\"" + ck + Properties.Resources.DoRegistered;
                }
            }
        }

        private void textBox1_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar != (char)Keys.Return) {
                Debug.WriteLine("textBox1_KeyPress: Text is not set\n");
                return;
            }

            if (this.textBox1.Text.Length <= 0) {
                Debug.WriteLine("textBox1_KeyPress: Text is not set\n");
                return;
            }

            // string originalPicture = this.wallpaper.pictureFile;
            int rt = this.wallpaper.ChangeDirectWallpaper(this.textBox1.Text);
            if (rt == 0) {
                display_PictureChanged(this.wallpaper.displayNameOfPictureFile);
            } else {
                display_PictureChanged(null);
            }
        }

        private void button8_Click(object sender, EventArgs e)
        {
            string fld;

            // This setting limits the folder to access from the dialog. If you want to set any folder, uncomment the next line.
            this.openFileDialog1.InitialDirectory = Environment.GetFolderPath(this.wallpaper.folderRoot == 0 ? Environment.SpecialFolder.MyPictures : Environment.SpecialFolder.MyComputer);
            Debug.WriteLine("button8_Click: " + this.openFileDialog1.InitialDirectory + "\n");
            DialogResult drt = this.openFileDialog1.ShowDialog();
            if (drt == DialogResult.OK) {
                fld = this.openFileDialog1.FileName;
                if (File.Exists(fld)) {
                    int rt = this.wallpaper.ChangeDirectWallpaper(fld);
                    if (rt == 0) {
                        display_PictureChanged(this.wallpaper.displayNameOfPictureFile);
                    } else {
                        display_PictureChanged(null);
                    }
                } else {
                    this.toolStripStatusLabel1.Text = Properties.Resources.ChangePicture_NG;
                }
            }
        }

        private void trackBar1_Scroll(object sender, EventArgs e)
        {
            int val;
            if (this.wallpaper.webLocalRatio == this.trackBar1.Value) {
                Debug.WriteLine("this.trackBar1.Value: value is not changed\n");
                return;
            }
            val = this.trackBar1.Value * 25;
            this.label8.Text = String.Format("{0,3:G} %", val);

            this.wallpaper.webLocalRatio = this.trackBar1.Value;
            this.wallpaper.configIsChanged = true;
            this.toolStripStatusLabel1.Text = Properties.Resources.WebRatioSet_0 + val.ToString() + Properties.Resources.WebRatioSet_1;

        }

        private void OptionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.optionsetting = new OptionSetting();
            this.optionsetting.sequentialMode = this.wallpaper.sequentialMode;
            this.optionsetting.folderRoot = this.wallpaper.folderRoot;
            this.optionsetting.continueAfterDirectChange = this.wallpaper.continueAfterDirectChange;
            this.optionsetting.folderRecursive = this.wallpaper.folderRecursive;
            this.optionsetting.Show();
            this.optionsetting.FormClosing += new FormClosingEventHandler(this.optionsetting_FormClosing);
        }

        private void optionsetting_FormClosing(object sender, FormClosingEventArgs e)
        {
            Debug.WriteLine("optionsetting_FormClosing:");
            if (this.optionsetting.isCanceled) {
                return;
            }
            this.wallpaper.sequentialMode = this.optionsetting.sequentialMode;
            this.wallpaper.folderRoot = this.optionsetting.folderRoot;
            this.wallpaper.continueAfterDirectChange = this.optionsetting.continueAfterDirectChange;
            this.wallpaper.folderRecursive = this.optionsetting.folderRecursive;
            this.wallpaper.configIsChanged = true;
            string smde, fmde;
            smde = this.wallpaper.sequentialMode == 0 ? (Properties.Resources.seq_Random) : (Properties.Resources.seq_Sequencial);
            fmde = this.wallpaper.folderRoot == 0 ? (Properties.Resources.myPicture) : (Properties.Resources.myComputer);
            this.toolStripStatusLabel1.Text = Properties.Resources.seq_Change_0 + smde + Properties.Resources.seq_Change_1 + fmde + Properties.Resources.seq_Change_2;
        }

        // service function of displaying after the picture changed.
        public void display_PictureChanged(string pictureName)
        {
            if (pictureName == null || pictureName.Length <= 0) {
                this.toolStripStatusLabel1.Text = Properties.Resources.ChangePicture_NG;
                return;
            }

            char[] spr = { '/', '\\' };
            string dstr = pictureName;
            dstr = dstr.Trim();
            dstr = dstr.Replace('\n', '_');
            dstr = dstr.Replace('\t', '_');
            dstr = dstr.Replace(' ', '_');
            string str = dstr.TrimEnd(spr);
            int at = str.LastIndexOfAny(spr);
            if (at >= 0) {
                str = str.Remove(0, at + 1);
            }
            this.label_CurrentPictureName.Text = dstr;
            this.toolStripStatusLabel1.Text = str + Properties.Resources.ChangePicture_Done;
            string ss = "photostand\n " + str;
            if (ss.Length < 64) {
                this.notifyIcon1.Text = "photostand\n " + str;
            }
        }

        private void radioButton1_CheckedChanged(object sender, EventArgs e)
        {
            this.changeDisplayStyle(0);
        }

        private void radioButton2_CheckedChanged(object sender, EventArgs e)
        {
            this.changeDisplayStyle(1);
        }

        private void radioButton3_CheckedChanged(object sender, EventArgs e)
        {
            this.changeDisplayStyle(2);
        }

        private void radioButton4_CheckedChanged(object sender, EventArgs e)
        {
            this.changeDisplayStyle(3);
        }

        private void radioButton5_CheckedChanged(object sender, EventArgs e)
        {
            this.changeDisplayStyle(4);
        }

        private void ToggleProcedureActive()
        {
            if (this.isActive) {
                this.isActive = false;
                this.timer1.Stop();
                this.SuspendSToolStripMenuItem.Text = Properties.Resources.Resume + "(&S)";
                this.notifyIcon1.Icon = photostand.Properties.Resources.photostand_inactive;
                this.button7.Text = Properties.Resources.Resume;
            } else {
                this.isActive = true;
                int min = Photostand.Index2Minutes(this.comboBox1.SelectedIndex);
                if (min > 0) {
                    this.timer1.Interval = min * 60 * 1000;
                    this.timer1.Start();
                } else {
                    this.timer1.Stop();
                }
                this.SuspendSToolStripMenuItem.Text = Properties.Resources.Suspend + "(&S)";
                this.notifyIcon1.Icon = photostand.Properties.Resources.photostand_active;
                this.button7.Text = Properties.Resources.Suspend;
            }
        }

        private void SuspendSToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ToggleProcedureActive();
        }

        private void button7_Click(object sender, EventArgs e)
        {
            ToggleProcedureActive();
        }

        private void EditResetRToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // Reset procedure.
            // Remove the two files, log and config, and restart the program.
            // [Todo]: Confirmation dialog may be necessary before resetting the configuration.
            DialogResult dr = MessageBox.Show(Properties.Resources.ResetConfirm, Properties.Resources.Reset, MessageBoxButtons.YesNo, MessageBoxIcon.Question);
            if (dr != DialogResult.Yes) {
                return;
            }
            wallpaper.ResetConfig();
            Application.Exit();
            System.Diagnostics.Process.Start(Application.ExecutablePath, null);

            // Restart current application, with same arguments/parameters
            //string[] args = Environment.GetCommandLineArgs();
            //for (int i = 1; i < args.Length; i++) // args[0] is always exe path/filename
            //    arguments += args[i] + " ";
            //Application.Exit();
            //System.Threading.Thread.Sleep(5000);
            //System.Diagnostics.Process.Start(Application.ExecutablePath, arguments);
        }
    } // class Form
} // namespace photostand
