﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.IO;
using System.Diagnostics;
using System.Xml;

namespace photostand
{
    class Webpicture
    {
        public string webPictureFile;      // Temporary picture file gotten from the wallpaper web server.
        public string baseURI;
        public string pictureUri;
        public string keyword;              // keyword for getting web pictures.

        private const int WebRetryMax = 3;  // retry getting webpictures when having errors.

        public Webpicture()
        {
            string appDir = String.Format(@"{0}\{1}\{2}", Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "Nikep", "photostand");
            if (!Directory.Exists(appDir)) {
                Directory.CreateDirectory(appDir);
            }
            //this.webPictureFile = String.Format(@"{0}\{1}", appDir, "_wwwtmp");
            this.webPictureFile = String.Format(@"{0}\{1}", appDir, "_wwwtmp.jpg");

            // If debug mode, the server should be local.
            string[] args = Environment.GetCommandLineArgs();
            foreach (string s in args) {
                switch (s) {
                    case "-Debug":
                        this.baseURI = "http://nociw.localdomain/cgi-bin/photostandweb.cgi";
                        break;
                    default:
                        this.baseURI = "http://www.nikep.net/cgi-bin/photostandweb.cgi";
                        break;
                }
            }
        }

        /// <summary>
        /// Get a wallpaper picture file from the photostand web server.
        /// </summary>
        public int WebGetPicture()
        {
            WebClient client = new WebClient();
            bool errorOccurred = false;
            int ntry = 0;

            // Add a user agent header in case the 
            // requested URI contains a query.
            client.Headers.Add("user-agent", "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.2; .NET CLR 1.0.3705;)");

            // create URI with the keyword.
            string uri = baseURI + "?keyword=" + this.keyword;

            do {
                // Send request and get the response data.
                try {
                    Stream data = client.OpenRead(uri);
                    StreamReader reader = new StreamReader(data);
                    //  string s = reader.ReadToEnd();
                    // Debug.WriteLine(s);

                    XmlReader xr = XmlReader.Create(reader);
                    while (xr.Read()) {
                        if (xr.NodeType == XmlNodeType.Element) {
                            if (xr.LocalName == "picture") {
                                this.pictureUri = xr.ReadString().Trim();
                                Debug.WriteLine("WebPictureName = " + this.pictureUri);
                            }
                        }
                    }
                    xr.Close();
                    data.Close();
                    reader.Close();

                    // retrieve the extenstion of the file
                    // string ext = ".jpg";
                    // this.webPictureFile += ext;

                    // add an basic referer of the uri
                    string[] rfs = this.pictureUri.Split(new Char[] { '/' });
                    if (rfs.Length >= 3) {
                        string referer = rfs[0] + "//" + rfs[2] + "/";
                        client.Headers.Add("Referer", referer);
                    }

                    // download the picture.
                    client.DownloadFile(this.pictureUri, this.webPictureFile);
                } catch (Exception ex) {
                    Debug.WriteLine("SaveLog: " + ex.ToString());
                    errorOccurred = true;
                }
            } while (errorOccurred && ntry++ < WebRetryMax);

            return errorOccurred ? 1 : 0;
        }
    }
}
